/*
**********************************************************************************************************************
*											        eGon
*						           the Embedded GO-ON Bootloader System
*									       eGON arm boot sub-system
*
*						  Copyright(C), 2006-2010, SoftWinners Microelectronic Co., Ltd.
*                                           All Rights Reserved
*
* File    : nand_boot1_write.c
*
* By      : Jerry
*
* Version : V2.00
*
* Date	  :
*
* Descript:
**********************************************************************************************************************
*/
#include "nand_flash.h"


__s32  Nand_Load_Boot1(__u32 Boot1_buf, __u32 buf_length )
{
    __u32  i;
    __s32  status;
    __u32 length;
    __u32 read_blks;
	boot_file_head_t  *bfh;

	NF_open( );                         // nand flash
	__msg("Succeed in opening nand flash.\n");

    for( i = BOOT1_START_BLK_NUM;  i <= BOOT1_LAST_BLK_NUM;  i++ )
    {
    	if( NF_read_status( i ) == NF_BAD_BLOCK )		// ǰǻ飬һ
            continue;

        /* 뵱ǰǰ512ֽڵݵSRAMУĿǻȡļͷ */
        if( NF_read( i << ( NF_BLK_SZ_WIDTH - NF_SCT_SZ_WIDTH ), (void *)Boot1_buf, 1 )  == NF_OVERTIME_ERR )
			continue;
		__msg("Succeed in reading Boot1 file head.\n");

		/* 쿴Ƿļͷ */
		if( check_magic( (__u32 *)Boot1_buf, BOOT1_MAGIC ) != 0 )
		{
			__inf("ERROR! block %u doesn't store head of Boot1 copy.\n", i );
			continue;
		}

		bfh = (boot_file_head_t *) Boot1_buf;
        length =  bfh->length;
        if( ( length & ( NF_SECTOR_SIZE - 1 ) ) != 0 )     // lengthNF_SECTOR_SIZE
        	continue;

		__inf("The size of Boot1 is %x.\n", length );

        if( length <=  NF_BLOCK_SIZE )
        {
        	/* һBoot1ı */
        	status = load_and_check_in_one_blk( i, (void *)Boot1_buf, length, NF_BLOCK_SIZE, BOOT1_MAGIC );
        	if( status == ADV_NF_OVERTIME_ERR )            // 
        		continue;
        	else if( status == ADV_NF_OK )
        	{
                __inf("Check is correct.\n");
                NF_close( );                        // رnand flash

                return 0;
            }
        }
        else
        {
        	/* ӶһBoot1ı */
        	status = load_in_many_blks( i, BOOT1_LAST_BLK_NUM, (void*)Boot1_buf,
        								length, NF_BLOCK_SIZE, &read_blks );
        	if( status == ADV_NF_LACK_BLKS )        // 
        	{
        		NF_close( );                        // رnand flash

        		return -1;
        	}
        	else if( status == ADV_NF_OVERTIME_ERR )
        		continue;

            if( check_sum( (__u32 *)Boot1_buf, length ) == 0 )
            {
                __inf("The file stored in start block %u is perfect.\n", i );
                NF_close( );                        // رnand flash

                return 0;
            }
        }
    }


	__inf("Can't find a good Boot1 copy in nand.\n");
    NF_close( );                        // رnand flash
    __inf("Ready to quit \"load_Boot1_from_nand\".\n");

    return -1;

}








